<?php

namespace Drupal\template_entities\EntityQuery;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\template_entities\TemplateManagerInterface;

/**
 * Provides template-specific helpers for altering entity queries.
 */
trait QueryTrait {

  /**
   * The template manager.
   *
   * @var \Drupal\template_entities\TemplateManagerInterface
   */
  protected $templateManager;

  /**
   * Constructs a Query object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param string $conjunction
   *   - AND: all of the conditions on the query need to match.
   *   - OR: at least one of the conditions on the query need to match.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection to run the query against.
   * @param array $namespaces
   *   List of potential namespaces of the classes belonging to this query.
   * @param \Drupal\template_entities\TemplateManagerInterface $template_manager
   *   The template manager.
   */
  public function __construct(EntityTypeInterface $entity_type, $conjunction, Connection $connection, array $namespaces, TemplateManagerInterface $template_manager) {
    parent::__construct($entity_type, $conjunction, $connection, $namespaces);

    $this->templateManager = $template_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function prepare() {
    parent::prepare();

    // Deal with entity references. By adding the allow_templates settings
    // to the selection handler settings, we ensure that the entity reference
    // field - whatever it's type - will allow selection of entities used as
    // templates. Only needed for the template entity form.
    /** @var \Drupal\Core\Entity\Plugin\EntityReferenceSelection\DefaultSelection $handler */
    if ($handler = $this->getMetadata('entity_reference_selection_handler')) {
      $configuration = $handler->getConfiguration();
      if (!empty($configuration['allow_templates'])) {
        $this->addTag('template_entities_allow_templates');
        $this->sqlQuery->addTag('template_entities_allow_templates');
        return $this;
      }
    }

    // Only apply query modification to entity queries for entity type/bundles
    // that may be used as templates.
    $template_types = $this->templateManager->getTemplateTypesForEntityType($this->entityType->id());
    if (empty($template_types)) {
      // No need to add filtering if this entity type is not used.
      return $this;
    }

    // Instances where the entity query is called but query modification
    // should not occur.
    if (!$this->accessCheck) {
      return $this;
    }

    // Instances where the entity query is called but query modification
    // should occur.
    // * listing.
    // or do we use permissions.

    // Don't alter all revision queries for now to allow access to revisions listings
    // of template content.
    if (!empty($this->allRevisions)) {
      return $this;
    }

    $id_field = $this->entityType->getKey('id');

    /** @var \Drupal\Core\Database\Query\SelectInterface $query */
    $query =& $this->sqlQuery;

    // Left join to the template entity reference field table to link entities
    // with references from template entities that have for template types
    // (template bundles) that apply to the entity type.
    $query->leftJoin('template__template_entity_id', 'template__template_entity_id', "%alias.template_entity_id_target_id = base_table.$id_field AND %alias.bundle IN (:types[])", [':types[]' => $template_types]);

    // Exclude all rows where there is a valid join to the template entity
    // reference field table.
    $query->isNull('template__template_entity_id.template_entity_id_target_id');

    return $this;
  }

}
