<?php

namespace Drupal\template_entities\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\template_entities\Plugin\TemplatePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class TemplateTypeForm.
 */
class TemplateTypeForm extends EntityForm {

  /**
   * The template plugin manager.
   *
   * @var \Drupal\template_entities\plugin\TemplatePluginManager
   */
  protected $manager;

  /**
   * The entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * @var \Drupal\template_entities\Entity\TemplateTypeInterface
   */
  protected $entity;

  /**
   * TemplateTypeForm constructor.
   *
   * @param \Drupal\template_entities\Plugin\TemplatePluginManager $manager
   *   The template plugin manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info service.
   */
  public function __construct(TemplatePluginManager $manager, EntityTypeBundleInfoInterface $entity_type_bundle_info) {
    $this->manager = $manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.template_plugin'),
      $container->get('entity_type.bundle.info')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $template_type = $this->entity;
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $template_type->label(),
      '#description' => $this->t("Label for the Template type."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $template_type->id(),
      '#machine_name' => [
        'exists' => '\Drupal\template_entities\Entity\TemplateType::load',
      ],
      '#disabled' => !$template_type->isNew(),
    ];

    $form['description'] = [
      '#title' => t('Description'),
      '#type' => 'textarea',
      '#default_value' => $template_type->getDescription(),
      '#description' => t('This text will be displayed on the <em>Add template</em> page.'),
    ];

    $options = [];
    foreach ($this->manager->getDefinitions() as $plugin_id => $plugin_definition) {
      $options[$plugin_id] = $plugin_definition['label'];
    }

    $form['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity type'),
      '#default_value' => $this->entity->get('type'),
      '#options' => $options,
      '#required' => TRUE,
      '#limit_validation_errors' => [['type']],
      '#submit' => ['::submitSelectType'],
      '#executes_submit_callback' => TRUE,
      '#ajax' => [
        'callback' => '::ajaxReplaceTypeOptionsForm',
        'wrapper' => 'type-options',
        'method' => 'replace',
      ],
    ];

    $form['type_options_container'] = [
      '#type' => 'container',
      '#prefix' => '<div id="type-options">',
      '#suffix' => '</div>',
    ];

    if ($this->entity->get('type')) {
      /** @var \Drupal\template_entities\Plugin\TemplatePluginInterface $type_plugin */
      $type_plugin = $this->entity->getTemplatePlugin();

      // Expose bundle conditions.
      if ($type_plugin->getDerivativeId() && $entity_type = $this->entityTypeManager->getDefinition($type_plugin->getDerivativeId())) {

        if ($entity_type->hasKey('bundle') && $bundles = $this->entityTypeBundleInfo->getBundleInfo($entity_type->id())) {
          $bundle_options = [];
          foreach ($bundles as $bundle_id => $bundle_info) {
            $bundle_options[$bundle_id] = $bundle_info['label'];
          }

          $form['type_options_container']['bundles'] = [
            '#title' => $entity_type->getBundleLabel(),
            '#type' => 'checkboxes',
            '#options' => $bundle_options,
            '#default_value' => $this->entity->get('bundles'),
            '#description' => $this->t('Check which bundles this template configuration type should be applied. Leave empty to allow any.'),
          ];
        }
      }
    }

    $form['add_action_link'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add action link to collection pages'),
      '#default_value' => $this->entity->get('add_action_link'),
      '#description' => $this->t("Check to have an action link added to the collection pages."),
    ];

    $form['collection_pages'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Override collection pages'),
      '#default_value' => $this->entity->get('collection_pages'),
      '#description' => $this->t("Add action links will be displayed on default collection pages. Override which pages will have add action links added by entering one path per line. An example path is /admin/content. Typically use this when using a view to list all nodes of a particular type - e.g. \"Pages\"."),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $template_type = $this->entity;
    $status = $template_type->save();


    switch ($status) {
      case SAVED_NEW:
        $this->messenger()
          ->addMessage($this->t('Created the %label Template type.', [
            '%label' => $template_type->label(),
          ]));
        break;

      default:
        $this->messenger()
          ->addMessage($this->t('Saved the %label Template type.', [
            '%label' => $template_type->label(),
          ]));
    }
    $form_state->setRedirectUrl($template_type->toUrl('collection'));
  }

  /**
   * Handles switching the type selector.
   *
   * @param $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return mixed
   */
  public function ajaxReplaceTypeOptionsForm($form, FormStateInterface $form_state) {
    return $form['type_options_container'];
  }

  /**
   * Handles submit call when alias type is selected.
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
*/
  public function submitSelectType(array $form, FormStateInterface $form_state) {
    $this->entity = $this->buildEntity($form, $form_state);
    $form_state->setRebuild();
  }

  public function buildEntity(array $form, FormStateInterface $form_state) {
    $entity = parent::buildEntity($form, $form_state);

    // Set bundles to NULL if none selected - @see DefaultSelection.
    $bundles = array_filter((array) $form_state->getValue('bundles'));
    if (empty($bundles)) {
      $entity->set('bundles', NULL);
    }
    return $entity;
  }

}
