<?php

namespace Drupal\template_entities\Plugin\TemplatePlugin;

use Drupal;
use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Database\Query\AlterableInterface;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Entity\EntityChangedInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\RevisionLogInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginWithFormsTrait;
use Drupal\Core\Routing\Router;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\template_entities\Plugin\TemplatePluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\RouteCollection;

/**
 * Base class for Template plugin plugins.
 *
 * @TemplatePlugin(
 *   id = "canonical_entities",
 *   deriver =
 *   "\Drupal\template_entities\Plugin\Deriver\EntityTemplatePluginDeriver"
 * )
 */
class TemplatePluginBase extends PluginBase implements TemplatePluginInterface, ContainerFactoryPluginInterface {

  use PluginWithFormsTrait;
  use StringTranslationTrait;

  /**
   * The plugin entity type.
   *
   * @var \Drupal\Core\Entity\EntityTypeInterface
   */
  protected EntityTypeInterface $entityType;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The router.
   *
   * @var \Drupal\Core\Routing\Router
   */
  protected Router $router;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeInterface $entity_type, EntityTypeManagerInterface $entity_type_manager, Router $router) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityType = $entity_type;
    $this->entityTypeManager = $entity_type_manager;
    $this->router = $router;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager')
        ->getDefinition($plugin_definition['entity_type_id']),
      $container->get('entity_type.manager'),
      $container->get('router.no_access_checks')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function duplicateEntity(EntityInterface $entity) {
    // Simple use.
    $duplicate = $entity->createDuplicate();

    $request_time = Drupal::time()
      ->getRequestTime();

    if ($entity instanceof EntityChangedInterface) {
      /** @var EntityChangedInterface $duplicate */
      $duplicate->setChangedTime($request_time);
    }

    if ($entity instanceof RevisionLogInterface) {
      /** @var RevisionLogInterface $duplicate */
      $duplicate->setRevisionLogMessage($this->t('Created from template.'));
    }

    return $duplicate;
  }

  /**
   * {@inheritdoc}
   */
  public function getCollectionLinkTemplate() {
    return $this->entityType->hasLinkTemplate('collection') ? $this->entityType->getLinkTemplate('collection') : Url::fromRoute('system.admin_content')
      ->toString();
  }

  /**
   * {@inheritdoc}
   */
  public function getCollectionRoute(RouteCollection $route_collection) {
    $collection_route_collection = new RouteCollection();

    if ($route = $route_collection->get("entity.{$this->getEntityTypeId()}.collection")) {
      $collection_route_collection->add("entity.{$this->getEntityTypeId()}.collection", $route);
    }
    elseif ($route = $route_collection->get("{$this->getEntityTypeId()}.collection")) {
      $collection_route_collection->add("{$this->getEntityTypeId()}.collection", $route);
    }
    else {
      $collection_route_collection->add('system.admin_content', $route_collection->get('system.admin_content'));
    }

    return $collection_route_collection;
  }

  /**
   * Returns the entity type ID.
   *
   * @return string
   *   The entity type ID.
   */
  protected function getEntityTypeId() {
    return $this->getDerivativeId();
  }

  /**
   * {@inheritdoc}
   */
  public function queryAlter(AlterableInterface $query, array $template_types) {
  }

  /**
   * Add template filter.
   *
   * @param \Drupal\Core\Database\Query\SelectInterface $query
   * @param array $template_types
   * @param string|null $base_table
   * @param string|null $id_field
   */
  protected function selectAddTemplateFilter(SelectInterface $query, array $template_types, string $base_table = NULL, string $id_field = NULL) {
    if (!$base_table) {
      $base_table = $base_table ?: $this->entityType->getBaseTable();
      if (!isset($query->getTables()[$base_table])) {
        foreach ($query->getTables() as $table) {
          if ($table['table'] === $base_table) {
            $base_table = $table['alias'];
          }
        }
      }
    }
    $id_field = $id_field ?: $this->entityType->getKey('id');

    // Left join to the template entity reference field table to link entities
    // with references from template entities that have for template types
    // (template bundles) that apply to the entity type.
    $query->leftJoin('template__template_entity_id', 'template__template_entity_id', "%alias.template_entity_id_target_id = $base_table.$id_field AND %alias.bundle IN (:types[])", [':types[]' => array_keys($template_types)]);

    // Exclude all rows where there is a valid join to the template entity
    // reference field table.
    $query->isNull('template__template_entity_id.template_entity_id_target_id');
  }

}
