<?php

namespace Drupal\template_entities;

use Drupal;
use Drupal\Core\Database\Query\AlterableInterface;
use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\template_entities\Entity\TemplateType;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides the template manager.
 */
class TemplateManager implements TemplateManagerInterface {

  use StringTranslationTrait;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The class resolver.
   *
   * @var \Drupal\Core\DependencyInjection\ClassResolverInterface
   */
  protected $classResolver;

  /**
   * Constructs a new WorkspaceManager.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\DependencyInjection\ClassResolverInterface $class_resolver
   *   The class resolver.
   */
  public function __construct(RequestStack $request_stack, EntityTypeManagerInterface $entity_type_manager, AccountProxyInterface $current_user, StateInterface $state, LoggerInterface $logger, ClassResolverInterface $class_resolver) {
    $this->requestStack = $request_stack;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->state = $state;
    $this->logger = $logger;
    $this->classResolver = $class_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public function isEntityTypeTemplateable(string $entity_type_id) {
    return $entity_type_id !== 'template' && !empty($this->getTemplateTypesForEntityType($entity_type_id));
  }

  /**
   * {@inheritdoc}
   */
  public function getTemplateTypesForEntityType(string $entity_type_id = NULL) {
    $template_types_by_entity_type = &drupal_static(__FUNCTION__);

    if ($template_types_by_entity_type === NULL) {
      $template_types_by_entity_type = [];
      $template_types = TemplateType::loadMultiple();

      /** @var TemplateType $template_type */
      foreach ($template_types as $template_type) {
        $target_entity_type_id = $template_type->getTargetEntityTypeId();
        if (!isset($template_types_by_entity_type[$target_entity_type_id])) {
          $template_types_by_entity_type[$target_entity_type_id] = [];
        }

        $template_types_by_entity_type[$target_entity_type_id][] = $template_type->id();
      }
    }

    if ($entity_type_id === NULL) {
      return $template_types_by_entity_type;
    }
    elseif (isset($template_types_by_entity_type[$entity_type_id])) {
      return $template_types_by_entity_type[$entity_type_id];
    }
    else {
      return [];
    }
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function isTemplate(EntityInterface $entity) {
    return !empty($this->getTemplateIdsForEntity($entity));
  }

  /**
   * Get template ids that use a given entity as a template.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return array|int
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getTemplateIdsForEntity(EntityInterface $entity) {
    $template_storage = $this->entityTypeManager->getStorage('template');
    $query = $template_storage->getQuery();
    $query->condition('template_entity_id', $entity->id());

    $template_types = $this->getTemplateTypesForEntityType($entity->getEntityType()
      ->id());
    $query->condition('type', $template_types, 'IN');

    return $query->execute();
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getTemplatesForEntity(EntityInterface $entity) {
    $template_storage = $this->entityTypeManager->getStorage('template');
    return $template_storage->loadMultiple($this->getTemplateIdsForEntity($entity));
  }

  /**
   * {@inheritdoc}
   */
  public function alterQuery(AlterableInterface $query) {
    // Efficiently call alter hooks for the query.
    // For all active template plugins, i.e. those that
    $template_types = TemplateType::loadMultiple();

    $active_plugins = [];
    $active_template_types = [];

    /** @var TemplateType $template_type */
    foreach ($template_types as $template_type) {
      $plugin = $template_type->getTemplatePlugin();
      $plugin_id = $plugin->getPluginId();
      $active_plugins[$plugin_id] = $plugin;

      if (isset($active_template_types[$plugin_id])) {
        $active_template_types[$plugin_id][$template_type->id()] = $template_type;
      }
      else {
        $active_template_types[$plugin_id] = [$template_type->id() => $template_type];
      }
    }

    $route_match = Drupal::routeMatch()->getRouteName();
    if ($route_match != 'entity.template.collection') {
      /** @var \Drupal\template_entities\Plugin\TemplatePluginInterface $template_plugin */
      foreach ($active_plugins as $plugin_id => $template_plugin) {
        $template_plugin->queryAlter($query, $active_template_types[$plugin_id]);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getTemplatesForEntityType(string $entity_type_id) {
    $template_type_ids = $this->getTemplateTypesForEntityType($entity_type_id);

    if (!empty($template_type_ids)) {
      $template_storage = $this->entityTypeManager->getStorage('template');
      return $template_storage->loadByProperties(['type' => $template_type_ids]);
    }

    return [];
  }

}
